/* ocaml-clanlib: ocaml bindings to the clanlib
 Copyright (C) 2013 Florent Monnier (monnier dot florent on gmail)
 Code provided under MIT license:
 See the file MIT_License.txt or this URL:
 http://opensource.org/licenses/mit-license.php
 This software is provided "AS IS", without warranty of any kind. */

#include <ClanLib/Display/Render/texture.h>
#include "cl_caml_incs.hpp"
#include "cl_caml_conv.hpp"
#include "cl_caml_gen.hpp"
#include "CL_Texture_stub.hpp"
#include "CL_GraphicContext_stub.hpp"
#include "CL_File_stub.hpp"

CAMLextern_C value
caml_CL_Texture_init(value gc, value width, value height)
{
    CL_Texture *tex = new CL_Texture(
            *CL_GraphicContext_val(gc), Int_val(width), Int_val(height));
    return Val_CL_Texture(tex);
}

CAMLextern_C value
caml_CL_Texture_read_file(value gc, value fullname)
{
    CL_Texture *tex = new CL_Texture(
            *CL_GraphicContext_val(gc), CL_StringRef_val(fullname));
    return Val_CL_Texture(tex);
}

CAMLextern_C value
caml_CL_Texture_of_file(value gc, value file, value image_type)
{
    CL_Texture *tex = new CL_Texture(
            *CL_GraphicContext_val(gc),
            *CL_File_val(file), CL_String_val(image_type));
    return Val_CL_Texture(tex);
}

CAMLextern_C value
caml_CL_Texture_delete(value tex)
{
    delete CL_Texture_val(tex);
    nullify_ptr(tex);
    return Val_unit;
}

// CL_Texture(CL_GraphicContext &context,
//       int width, int height, CL_TextureFormat internal_format = cl_rgba);

// CL_Texture(CL_GraphicContext &context,
//       const CL_StringRef &filename, const CL_VirtualDirectory &directory);

// vim: sw=4 sts=4 ts=4 et
