/* ocaml-clanlib: ocaml bindings to the clanlib
 Copyright (C) 2013 Florent Monnier (monnier dot florent on gmail)
 Code provided under MIT license:
 See the file MIT_License.txt or this URL:
 http://opensource.org/licenses/mit-license.php
 This software is provided "AS IS", without warranty of any kind. */

#include <ClanLib/Core/Math/vec3.h>
#include "cl_caml_incs.hpp"
#include "cl_caml_conv.hpp"
#include "CL_Vec3f_stub.hpp"

CAMLextern_C value
caml_CL_Vec3f_init(value v1, value v2, value v3)
{
    CL_Vec3f *vec = new CL_Vec3f(
            Float_val(v1),
            Float_val(v2),
            Float_val(v3));
    return Val_CL_Vec3f(vec);
}

CAMLextern_C value
caml_CL_Vec3f_delete(value vec)
{
    delete CL_Vec3f_val(vec);
    nullify_ptr(vec);
    return Val_unit;
}

CAMLextern_C value
caml_CL_Vec3f_print(value vec)
{
    float x = CL_Vec3f_val(vec)->x;
    float y = CL_Vec3f_val(vec)->y;
    float z = CL_Vec3f_val(vec)->z;
    std::cout << "Vec3f: "
        << x << " "
        << y << " "
        << z << std::endl;
    return Val_unit;
}

CAMLextern_C value
caml_CL_Vec3f_normalize(value vec)
{
    CL_Vec3f *norm = new CL_Vec3f(
        CL_Vec3f::normalize(
            *CL_Vec3f_val(vec)));
    return Val_CL_Vec3f(norm);
}

CAMLextern_C value
caml_CL_Vec3f_dot(value vec1, value vec2)
{
    float r = CL_Vec3f::dot(
            *CL_Vec3f_val(vec1),
            *CL_Vec3f_val(vec2));
    return caml_copy_double(r);
}

CAMLextern_C value
caml_CL_Vec3f_cross(value vec1, value vec2)
{
    CL_Vec3f *r = new CL_Vec3f(
        CL_Vec3f::cross(
            *CL_Vec3f_val(vec1),
            *CL_Vec3f_val(vec2)));
    return Val_CL_Vec3f(r);
}

CAMLextern_C value
caml_CL_Vec3f_length(value vec)
{
    float len = CL_Vec3f_val(vec)->length();
    return caml_copy_double(len);
}

CAMLextern_C value
caml_CL_Vec3f_distance(value vec1, value vec2)
{
    float dist = CL_Vec3f_val(vec1)->distance(
            *CL_Vec3f_val(vec2));
    return caml_copy_double(dist);
}

// vim: sw=4 sts=4 ts=4 et
